package com.sample;

import java.util.ArrayList;

import android.app.IntentService;
import android.content.ComponentName;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.net.Uri.Builder;
import android.util.Log;

import com.sample.PluginConstants.ServiceIntentCmd;

public class SampleService extends IntentService {

	private int mSourceId;

	public SampleService() {
		super("SampleService");
	}

	@Override
	protected void onHandleIntent(Intent intent) {
		mSourceId = getSourceId();
		if (intent != null) {
			String serviceCommand = intent.getStringExtra(ServiceIntentCmd.SERVICE_COMMAND_KEY);
			if (PluginConstants.ServiceIntentCmd.REGISTER_PLUGIN.equals(serviceCommand)) {
				registerOrUpdatePlugin();
			} else if (ServiceIntentCmd.REFRESH_REQUEST.equals(serviceCommand)) {
				if (mSourceId != -1) {
					if (isSourceEnabled()) {
						registEvent();
					}
				}
			} else {
				// 何も登録されていないとき
			}
		}
	}

	private void registerOrUpdatePlugin() {
		// プラグインの登録/更新
		insertOrUpdatePlugin();

		// ソースの登録/更新
		insertOrUpdateSource();
	}

	private void insertOrUpdateSource() {
		Cursor cursor = null;
		try {

			ContentValues values = new ContentValues();
			values.putNull("current_status");
			values.put("name", "Sample");// 設定画面等で使用される
			Builder iconUriBuilder = new Uri.Builder().scheme(ContentResolver.SCHEME_ANDROID_RESOURCE)
					.authority(getPackageName()).appendPath(Integer.toString(R.drawable.icon));
			values.put("icon_uri", iconUriBuilder.toString());

			cursor = getContentResolver().query(Uri.parse("content://com.sonyericsson.eventstream/sources"),
					new String[] { "_id" }, null, null, null);
			if (cursor != null && cursor.moveToFirst() && cursor.getCount() > 0) { 
				mSourceId = cursor.getInt(cursor.getColumnIndexOrThrow("_id"));
				int updated = getContentResolver().update(Uri.parse("content://com.sonyericsson.eventstream/sources"),
						values, null, null);
			} else {
				
				values.put("enabled", 1);
				Uri uri = getContentResolver().insert(Uri.parse("content://com.sonyericsson.eventstream/sources"),
						values);
				mSourceId = (int) ContentUris.parseId(uri);
			}
		} finally {
			if (cursor != null) {
				cursor.close();
			}
		}

	}

	private void insertOrUpdatePlugin() {
		Cursor cursor = null;
		try {
			cursor = getContentResolver().query(Uri.parse("content://com.sonyericsson.eventstream/plugins"), null,
					null, null, null);

			ContentValues values = new ContentValues();
			String configName = new ComponentName(getPackageName(), PluginConfigActivity.class.getName())
					.flattenToShortString();
			Builder iconUriBuilder = new Uri.Builder().scheme(ContentResolver.SCHEME_ANDROID_RESOURCE)
					.authority(getPackageName()).appendPath(Integer.toString(R.drawable.icon));

			values.put("name", "Sample plugin");
			values.put("icon_uri", iconUriBuilder.toString());
			values.put("api_version", 1);
			values.put("config_activity", configName);
			values.put("config_state", 2);
			values.put("config_text", "サンプル設定");
			values.put("status_support", 1);
			values.put("status_text_max_length", 10);

			if (cursor != null && cursor.moveToNext() && cursor.getCount() > 0) { 
				getContentResolver().update(Uri.parse("content://com.sonyericsson.eventstream/plugins"), values, null,
						null);
			} else {
				getContentResolver().insert(Uri.parse("content://com.sonyericsson.eventstream/plugins"), values);
			}
		} finally {
			if (cursor != null) {
				cursor.close();
			}
		}
	}

	private boolean isSourceEnabled() {
		int enabled;
		Cursor cursor = null;
		try {
			cursor = getContentResolver().query(Uri.parse("content://com.sonyericsson.eventstream/sources"),
					new String[] { "_id", "enabled" }, null, null, null);
			if (cursor != null && cursor.moveToFirst() && cursor.getCount() > 0) { 
				enabled = cursor.getInt(cursor.getColumnIndexOrThrow("enabled"));
			} else {
				enabled = 0;
			}
		} finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		if (enabled == 1) {
			return true;
		}

		return false;
	}

	private int getSourceId() {
		int sourceId;
		Cursor cursor = null;
		try {
			cursor = getContentResolver().query(Uri.parse("content://com.sonyericsson.eventstream/sources"),
					new String[] { "_id" }, null, null, null);
			if (cursor != null && cursor.moveToFirst() && cursor.getCount() > 0) { 
				sourceId = cursor.getInt(cursor.getColumnIndexOrThrow("_id"));
			} else {
				sourceId = -1;
			}
		} finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		return sourceId;
	}

	private int registEvent() {

		ContentResolver contentResolver = getContentResolver();
		ArrayList<ContentValues> eventstreamEventList = new ArrayList<ContentValues>();

		ArrayList<Item> list = getItems();
		int itemId = 0;
		for (Item data : list) {

			ContentValues values = new ContentValues();
			values.put("event_key", itemId);
			values.put("source_id", mSourceId);
			values.put("title", data.title);
			values.put("message", data.message);
			values.put("published_time", data.publishedTime);
			values.put("image_uri", data.imageUri);
			values.put("status_icon_uri", data.statusIconUri);
			values.put("outgoing", data.outgoing);
			values.put("personal", data.personal);

			eventstreamEventList.add(values);
			itemId++;
		}
		ContentValues[] bulkContentValues = new ContentValues[eventstreamEventList.size()];
		int contentValuesIndex = 0;
		for (ContentValues values : eventstreamEventList) {
			bulkContentValues[contentValuesIndex] = values;
			contentValuesIndex++;
		}
		return contentResolver
				.bulkInsert(Uri.parse("content://com.sonyericsson.eventstream/events"), bulkContentValues);

	}

	private ArrayList<Item> getItems() {
		ArrayList<Item> list = new ArrayList<Item>();

		Builder statusIconUriBuilder = new Uri.Builder().scheme(ContentResolver.SCHEME_ANDROID_RESOURCE)
				.authority(getPackageName()).appendPath(Integer.toString(R.drawable.check));

		// 1番目のアイテム
		Item item = new Item();
		item.imageUri = new Uri.Builder().scheme(ContentResolver.SCHEME_ANDROID_RESOURCE).authority(getPackageName())
				.appendPath(Integer.toString(R.drawable.s)).toString();
		item.title = "store.sonyericsson.jp";
		item.message = "http://store.sonyericsson.jp";
		item.publishedTime = System.currentTimeMillis();
		item.outgoing = 0;
		item.personal = 0;
		item.statusIconUri = statusIconUriBuilder.toString();

		// 2番目のアイテム
		Item item2 = new Item();
		item2.imageUri = new Uri.Builder().scheme(ContentResolver.SCHEME_ANDROID_RESOURCE).authority(getPackageName())
				.appendPath(Integer.toString(R.drawable.c)).toString();
		item2.title = "cyberspacefarm.com";
		item2.message = "http://cyberspacefarm.com/2011/0624/wb-db-press-63/";
		item2.publishedTime = System.currentTimeMillis();
		item2.outgoing = 0;
		item2.personal = 1;
		item2.statusIconUri = statusIconUriBuilder.toString();

		list.add(item);
		list.add(item2);

		return list;

	}

}
